"""Visualisations of Radio Galaxy Zoo subjects."""

import astropy.io.fits
import matplotlib.colors
import matplotlib.pyplot
import numpy

from . import rgz_data as data
from .config import config

def image(im, contrast=0.05):
    """Plots an RGZ image.

    im: NumPy array of an RGZ image.
    contrast: Log scale parameter, default 0.05.
    -> MatPlotLib image plot.
    """
    im = im - im.min() + contrast
    return matplotlib.pyplot.imshow(im, origin='lower', cmap='gray',
            norm=matplotlib.colors.LogNorm(vmin=im.min(), vmax=im.max()))

def clicks(cs, colour='gray'):
    """Plots a list of RGZ clicks.

    Clicks will be flipped and scaled to match the FITS images.

    cs: List of (x, y) click tuples.
    -> MatPlotLib scatter plot.
    """
    cs = (config['surveys']['atlas']['fits_height'] -
          numpy.array(cs) * config['surveys']['atlas']['click_to_fits'])
    return matplotlib.pyplot.scatter(cs[:, 0], cs[:, 1], color=colour)

def contours(subject, colour='gray'):
    """Plots the contours of a subject.

    subject: RGZ subject.
    colour: Colour to plot contours in. Default 'gray'.
    """
    for row in data.get_contours(subject)['contours']:
        for col in row:
            xs = []
            ys = []
            for pair in col['arr']:
                xs.append(pair['x'])
                ys.append(pair['y'])
            ys = config['surveys']['atlas']['fits_height'] - numpy.array(ys)
            matplotlib.pyplot.plot(xs, ys, c=colour)

def ir(subject):
    """Plots the IR image of a subject.

    subject: RGZ subject.
    -> MatPlotLib image plot.
    """
    return image(data.get_ir(subject))

def radio(subject):
    """Plots the radio image of a subject.

    subject: RGZ subject.
    -> MatPlotLib image plot.
    """
    return image(data.get_radio(subject))

def subject(s):
    """Shows the IR and contours of a subject.

    s: RGZ subject.
    """
    ir(s)
    contours(s, colour='green')
    matplotlib.pyplot.xlim(0, config['surveys']['atlas']['fits_width'])
    matplotlib.pyplot.ylim(0, config['surveys']['atlas']['fits_height'])



"""Trains a convolutional neural network.

Matthew Alger
The Australian National University
2016
"""

import argparse
import logging
import os.path

import astropy.io.fits
import h5py
import numpy

from .config import config

PATCH_RADIUS = config['patch_radius']
PATCH_DIAMETER = PATCH_RADIUS * 2


def train(training_h5, model_json, weights_path, epochs, batch_size):
    """Trains a CNN.

    training_h5: Training HDF5 file.
    model_json: JSON model file.
    weights_path: Output weights HDF5 file.
    epochs: Number of training epochs.
    batch_size: Batch size.
    """
    import keras.models
    model = keras.models.model_from_json(model_json.read())
    model.compile(loss='binary_crossentropy', optimizer='adadelta')

    train_set = training_h5['is_ir_train'].value
    ir_survey = training_h5.attrs['ir_survey']

    if ir_survey == 'swire':
        training_inputs = training_h5['features'].value[train_set, 6:]
    elif ir_survey == 'wise':
        training_inputs = training_h5['features'].value[train_set, 5:]

    training_inputs = training_inputs.reshape(
            (-1, 1, PATCH_DIAMETER, PATCH_DIAMETER))
    training_outputs = training_h5['labels'].value[train_set]
    assert training_inputs.shape[0] == training_outputs.shape[0]

    # Downsample for class balance.
    zero_indices = (training_outputs == 0).nonzero()[0]
    one_indices = (training_outputs == 1).nonzero()[0]
    subset_zero_indices = numpy.random.choice(zero_indices,
            size=(len(one_indices,)), replace=False)
    all_indices = numpy.hstack([subset_zero_indices, one_indices])
    all_indices.sort()

    training_inputs = training_inputs[all_indices]
    training_outputs = training_outputs[all_indices]
    assert (training_outputs == 1).sum() == (training_outputs == 0).sum()

    model.fit(training_inputs, training_outputs, batch_size=batch_size,
              nb_epoch=epochs)
    model.save_weights(weights_path, overwrite=True)


def _populate_parser(parser):
    parser.description = 'Trains a convolutional neural network.'
    parser.add_argument('--h5', default='data/training.h5',
                        help='HDF5 training data file')
    parser.add_argument('--model', default='data/model.json',
                        help='JSON model file')
    parser.add_argument('--output', default='data/weights.h5',
                        help='HDF5 file for output weights')
    parser.add_argument('--epochs', default=10,
                        help='number of epochs to train for')
    parser.add_argument('--batch_size', default=100, help='batch size')


def _main(args):
    with h5py.File(args.h5, 'r') as training_h5:
        with open(args.model, 'r') as model_json:
            train(training_h5, model_json, args.output, int(args.epochs),
                  int(args.batch_size))


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    _populate_parser(parser)
    args = parser.parse_args()
    _main(args)



"""Generates the Radio Galaxy Zoo catalogue."""

import contextlib
import logging
import sqlite3

import astropy.coordinates
import astropy.units
import astropy.wcs
import numpy

from . import config
from . import data
from .exceptions import CatalogueError

def distance_from(point):
    """Returns a function that returns the distance from a point.

    Used for keys.
    """
    def _dist(other):
        return numpy.hypot(point[0] - other[0], point[1] - other[1])
    return _dist

def make_host(subject, wcs, cache_name, consensus):
    """Returns an RGZ and SWIRE name for the host, and the location.

    subject: RGZ subject dict.
    wcs: World coordinate system (from Astropy).
    cache_name: Name of Gator cache.
    consensus: Consensus dict from database.
    -> (str: RGZ_JHHMMSS-DDMMSS, SWIRE name, RA, DEC)
    """
    # We want RGZ_JHHMMSS-DDMMSS and an associated SWIRE result. Let's start
    # with the SWIRE result so that the coordinates we get are accurate and
    # reproducible. Convert pixel coordinates into RA/DEC.
    x = consensus['source_x']
    # Pretty much following willettk's rgz-analysis code here.
    raise NotImplementedError('Scales have been changed and code not updated.')
    y = config.get('fits_image_height') - consensus['source_y']
    x, y = wcs.wcs_pix2world([x], [y], 1)

    # Get the closest SWIRE object.
    p_hosts = data.get_potential_hosts(subject, cache_name,
                                       convert_to_px=False)
    dist_func = distance_from((x, y))
    nearest = min(p_hosts, key=dist_func)

    # Cutoff is in degrees.
    if not dist_func(nearest) < config.get('swire_distance_cutoff'):
        raise CatalogueError(
            'Closest SWIRE object is not nearby for {}. '
            'Distance: {:.02} degrees'.format(
                    subject['zooniverse_id'],
                    float(dist_func(nearest))))

    swire_name = p_hosts[nearest]['name'].decode('ascii')

    # This provides our coordinates. Separate out HHMMSS/DDMMSS. We can do this
    # using astropy.coordinates.SkyCoord. Note that I'm using the clon/clat
    # values returned by Gator; this helps keep us lined up with the SWIRE
    # coordinates since if you use the RA/DEC raw you get rounding errors.
    clon = p_hosts[nearest]['clon'].decode('ascii')
    clat = p_hosts[nearest]['clat'].decode('ascii')
    coords = astropy.coordinates.SkyCoord(clon + ' ' + clat, unit='deg')
    ra = coords.ra.hms
    dec = coords.dec.signed_dms

    # Construct the name.
    rgz_name = ('RGZ_J{ra_h:02}{ra_m:02}{ra_s:04.01f}{sign}'
                '{dec_d:02}{dec_m:02}{dec_s:02}').format(
                ra_h=int(ra[0]),
                ra_m=int(ra[1]),
                ra_s=ra[2],
                sign='-' if dec[0] < 0 else '+',
                dec_d=int(dec[1]),
                dec_m=int(dec[2]),
                dec_s=int(round(dec[3])))

    return rgz_name, swire_name, clon, clat, consensus['location_agreement']

def generate(db_path, cache_name, consensus_table, host_table, radio_table,
             atlas=False):
    """Generates the Radio Galaxy Zoo catalogue.

    Warning: table arguments are not validated! This could be dangerous.

    db_path: Path to consensus database.
    cache_name: Name of Gator cache.
    consensus_table: Database table of consensuses.
    host_table: Output database table of RGZ hosts. Will be overwritten!
    radio_table: Output database table of RGZ radio sources. Will be
        overwritten!
    atlas: Whether to only freeze ATLAS subjects. Default False (though this
        function currently only works for True).
    """
    with contextlib.closing(sqlite3.connect(db_path)) as conn:
        conn.row_factory = sqlite3.Row
        cur = conn.cursor()

        cur.execute('DROP TABLE IF EXISTS {}'.format(host_table))
        cur.execute('DROP TABLE IF EXISTS {}'.format(radio_table))
        conn.commit()

        cur.execute('CREATE TABLE {} '
                    '(zooniverse_id TEXT, source TEXT, rgz_name TEXT, '
                    'swire_name TEXT, ra TEXT, dec TEXT, agreement REAL)'
                    ''.format(host_table))
        cur.execute('CREATE TABLE {} '
                    '(rgz_name TEXT, radio_component TEXT, agreement REAL)'
                    ''.format(radio_table))
        conn.commit()

        host_sql = ('INSERT INTO {} (zooniverse_id, source, rgz_name, '
                    'swire_name, ra, dec, agreement) VALUES '
                    '(?, ?, ?, ?, ?, ?, ?)'.format(host_table))
        radio_sql = ('INSERT INTO {} (rgz_name, radio_component, agreement) '
                     'VALUES (?, ?, ?)'.format(radio_table))

        host_params = []
        radio_params = []

        # Store how many hosts have no associated SWIRE object (for debugging).
        n_no_matches = 0

        # Store how many hosts have null consensuses (for debugging).
        n_null = 0

        n_subjects = data.get_all_subjects(atlas=atlas).count()
        for index, subject in enumerate(data.get_all_subjects(atlas=atlas)):
            print('Generating catalogue: {}/{} ({:.02%})'.format(
                    index + 1, n_subjects, (index + 1) / n_subjects), end='\r')
            consensuses = cur.execute(
                    'SELECT * FROM {} WHERE '
                    'zooniverse_id = ?'.format(consensus_table),
                    [subject['zooniverse_id']])

            fits = data.get_ir_fits(subject)
            wcs = astropy.wcs.WCS(fits.header)

            for consensus in consensuses:
                # Each consensus represents one AGN.
                if consensus['source_x'] and consensus['source_y']:
                    # Not null.
                    try:
                        rgz_name, swire_name, ra, dec, agreement = make_host(
                                subject, wcs, cache_name, consensus)
                    except CatalogueError:
                        logging.debug('No SWIRE object for %s (%.2f, %.2f).',
                                subject['zooniverse_id'], consensus['source_x'],
                                consensus['source_y'])
                        n_no_matches += 1
                        continue
                    host_params.append((subject['zooniverse_id'],
                                        subject.get('metadata', {}).get(
                                                'source'),
                                        rgz_name, swire_name, ra, dec,
                                        agreement))

                    # Get radio components.
                    radio_components = set(  # Set to nix duplicates.
                            consensus['radio_signature'].split(';'))
                    for radio_component in radio_components:
                        radio_params.append((rgz_name, radio_component,
                                             consensus['radio_agreement']))
                else:
                    n_null += 1
                    logging.debug('Skipping null consensus for subject %s.',
                                  subject['zooniverse_id'])

        logging.debug('%d hosts with no associated SWIRE object.', n_no_matches)
        logging.debug('%d hosts with null consensuses.', n_null)

        logging.debug('Writing to database.')
        cur.executemany(host_sql, host_params)
        cur.executemany(radio_sql, radio_params)
        conn.commit()

        # Go back and clear up duplicates. The process is as follows:
        # 1. Check the components table for duplicates.
        # 2. For each duplicate, we want to choose one "true" host. This is
        #    because each component can only belong to one host. We will pick
        #    the host with the highest percentage agreement (though there are
        #    likely better ways to do this).
        # 3. Delete the duplicates and replace them with a new component with
        #    the "true" host and the highest agreement.
        # 4. For each host that no longer has a component, delete it from the
        #    hosts table.
        all_duplicates = cur.execute("""select radio_component
                                        from {}
                                        group by radio_component
                                        having count(*) > 1""".format(
                                                radio_table))
        mur = conn.cursor()
        # Number of deleted radio components.
        n_deleted = 0
        for radio_component in all_duplicates:
            name = radio_component['radio_component']
            logging.debug('Removing duplicates for %s.', name)
            best = next(mur.execute("""select rgz_name, agreement
                                       from {}
                                       where radio_component = ?
                                       order by agreement desc
                                       limit 1""".format(radio_table), [name]))
            mur.execute("""delete from {}
                           where radio_component = ?""".format(radio_table),
                           [name])
            n_deleted += mur.rowcount
            mur.execute("""insert into {}
                           (rgz_name, radio_component, agreement)
                           values (?, ?, ?)""".format(radio_table),
                           [best['rgz_name'], name, best['agreement']])
        conn.commit()
        logging.debug('Deleted %d duplicate radio components.', n_deleted)

        logging.debug('Removing hosts with no components.')
        cur.execute("""delete from {0}
                       where
                            rgz_name in (
                                select {0}.rgz_name
                                from {0}
                                left join {1}
                                   on {0}.rgz_name = {1}.rgz_name
                                where {1}.rgz_name is null
                            )""".format(
                host_table, radio_table))
        conn.commit()
        logging.debug('Deleted %d duplicate hosts.', cur.rowcount)

        logging.debug('Removing duplicate hosts.')
        cur.execute("""delete from {0}
                       where rowid not in
                            (select min(rowid)
                             from {0}
                             group by rgz_name)""".format(host_table))
        conn.commit()



"""Classifier for radio subjects.

Matthew Alger
The Australian National University
2016
"""

import logging

import numpy
import sklearn.cross_validation
from sklearn.ensemble import RandomForestClassifier
import sklearn.linear_model
import sklearn.pipeline
import sklearn.preprocessing

from .config import config

ARCMIN = 1 / 60  # deg
IMAGE_SIZE = (config['surveys']['atlas']['fits_height']
             * config['surveys']['atlas']['fits_width'])


class RGZClassifier(object):
    
    def __init__(self, ir_features, n_astro,
                 Classifier=sklearn.linear_model.LogisticRegression):
        self.ir_features = ir_features
        self.n_astro = n_astro
        self._classifier = Classifier(class_weight='balanced')
        self._astro_transformer = sklearn.pipeline.Pipeline([
            ('normalise', sklearn.preprocessing.Normalizer()),
            ('scale', sklearn.preprocessing.StandardScaler()),
        ])
        self._image_transformer = sklearn.pipeline.Pipeline([
            ('normalise', sklearn.preprocessing.Normalizer()),
        ])
    
    def _fit_transformers(self, ir_indices):
        self._astro_transformer.fit(self.ir_features[ir_indices, :self.n_astro])
        self._image_transformer.fit(self.ir_features[ir_indices, self.n_astro:])
    
    def _transform(self, features):
        return numpy.hstack([
            self._astro_transformer.transform(features[:, :self.n_astro]),
            self._image_transformer.transform(features[:, self.n_astro:]),
        ])
    
    def train(self, ir_indices, ir_labels):
        ir_indices = list(ir_indices)
        self._fit_transformers(ir_indices)
        ir_features = self._transform(self.ir_features[ir_indices])
        self._classifier.fit(ir_features, ir_labels)
    
    def _predict(self, atlas_vector):
        # Split the ATLAS vector into its components.
        position = atlas_vector[:2]
        image = atlas_vector[2:2 + IMAGE_SIZE]
        distances = atlas_vector[2 + IMAGE_SIZE:]
        # Get nearby IR objects and their features.
        nearby_indices = list((distances < ARCMIN).nonzero()[0])
        ir_features = self._transform(self.ir_features[nearby_indices])
        # Find how likely each object is to be the host galaxy.
        probabilities = self._classifier.predict_proba(ir_features)[:, 1]
        return nearby_indices, probabilities
    
    def predict(self, atlas_vector):
        nearby_indices, probabilities = self._predict(atlas_vector)
        # Return the index of the most likely host galaxy.
        return nearby_indices[probabilities.argmax()]
    
    def predict_probabilities(self, atlas_vector):
        nearby_indices, probabilities = self._predict(atlas_vector)
        out_probabilities = numpy.zeros(self.ir_features.shape[0])
        out_probabilities[nearby_indices] = probabilities
        return out_probabilities


def generate_subset(p, indices):
    indices = indices.copy()
    numpy.random.shuffle(indices)
    return indices[:int(p * len(indices))]


class RGZCommittee(object):

    def __init__(self, n_committee, ir_features, n_astro, lr_frac):
        self.classifiers = [RGZClassifier(ir_features, n_astro)
                            for _ in range(round(lr_frac * n_committee))]
        self.classifiers += [RGZClassifier(ir_features, n_astro,
                                           Classifier=RandomForestClassifier)
                            for _ in range(round((1 - lr_frac) * n_committee))]
        assert len(self.classifiers) == n_committee

    def train(self, ir_indices, ir_labels):
        for classifier in self.classifiers:
            classifier.train(ir_indices, ir_labels)

    def k_fold_train_and_label(self, n_folds, subset_size, atlas_vectors,
                               ir_labels):
        folds = sklearn.cross_validation.KFold(atlas_vectors.shape[0],
                                               n_folds=n_folds,
                                               shuffle=True,
                                               random_state=0)
        classifications = numpy.zeros((atlas_vectors.shape[0],
                                       len(self.classifiers)))
        for fold_index, (train_indices, test_indices) in enumerate(folds):
            logging.debug('Testing fold {}/{}.'.format(fold_index + 1, n_folds))
            # Precompute which IR objects should be reserved for testing.
            # See later comment.
            test_ir_indices = set()
            for atlas_index in test_indices:
                distances = atlas_vectors[atlas_index, 2 + IMAGE_SIZE:]
                nearby_indices = (distances < ARCMIN).nonzero()[0]
                for ir_index in nearby_indices:
                    test_ir_indices.add(ir_index)

            for index, classifier in enumerate(self.classifiers):
                train_subset = generate_subset(subset_size, train_indices)
                # This subset is of ATLAS indicies, so we need to convert this
                # into IR indices. An IR object is in the training set if it is
                # nearby a training ATLAS object and *not* nearby a testing
                # ATLAS object. The easiest way to do this is to find all nearby
                # indices, and all testing indices, and then find the set
                # difference.
                train_ir_indices = set()
                for atlas_index in train_subset:
                    distances = atlas_vectors[atlas_index, 2 + IMAGE_SIZE:]
                    nearby_indices = (distances < ARCMIN).nonzero()[0]
                    for ir_index in nearby_indices:
                        train_ir_indices.add(ir_index)
                
                train_ir_indices -= test_ir_indices
                train_ir_indices = numpy.array(sorted(train_ir_indices))

                classifier.train(train_ir_indices, ir_labels[train_ir_indices])
                
                # Classify all the test subjects.
                for atlas_index in test_indices:
                    classification = classifier.predict(
                            atlas_vectors[atlas_index])
                    classifications[atlas_index, index] = classification

        return classifications



#!/usr/bin/env python3
"""
Repacks an HDF5 file.

Matthew Alger
The Australian National University
2016
"""

import argparse
import os
import shutil
import tempfile

import h5py


def _populate_parser(parser):
    parser.description = 'Repacks an HDF5 file.'
    parser.add_argument('h5', help='HDF5 file to repack')


def _main(args):
    with tempfile.TemporaryDirectory() as tempdir:
        temp_h5 = os.path.join(tempdir, 'temp.h5')
        with h5py.File(args.h5, 'r') as input_h5:
            with h5py.File(temp_h5, 'w') as output_h5:
                for key in input_h5:
                    output_h5[key] = input_h5[key].value

        os.remove(args.h5)
        shutil.copyfile(temp_h5, args.h5)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    _populate_parser(parser)
    args = parser.parse_args()
    _main(args)



"""Generates a raw features -> labels dataset.

This should be run after generate_training_data.py, but before
generate_cnn_outputs.py.

Outputs to HDF5 with structure:
/
    features
    labels

Matthew Alger
The Australian National University
2016
"""

import argparse

import h5py


def generate(f_h5, out_f_h5):
    out_f_h5.create_dataset('features', data=f_h5['features'])
    out_f_h5.create_dataset('labels', data=f_h5['labels'])


def _populate_parser(parser):
    parser.description = 'Generates a raw features -> labels dataset.'
    parser.add_argument('-i', default='data/training.h5',
                        help='HDF5 input (training) file')
    parser.add_argument('-o', default='data/dataset.h5',
                        help='HDF5 output file')


def _main(args):
    with h5py.File(args.i, 'r') as f_h5:
        with h5py.File(args.o, 'w') as out_f_h5:
            generate(f_h5, out_f_h5)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    _populate_parser(parser)
    args = parser.parse_args()
    _main(args)



"""Builds the convolutional neural network model.

Matthew Alger
The Australian National University
2016
"""

import argparse


def main(n_filters, conv_size, pool_size, dropout, hidden_layer_size,
         patch_size, out_path=None):
    # Imports must be in the function, or whenever we import this module, Keras
    # will dump to stdout.
    import keras.layers.core as core
    import keras.layers.convolutional as conv
    import keras.models as models

    model = models.Sequential()

    model.add(conv.Convolution2D(n_filters, conv_size, conv_size,
                                 border_mode='valid',
                                 input_shape=(1, patch_size, patch_size)))
    model.add(core.Activation('relu'))
    model.add(conv.MaxPooling2D(pool_size=(pool_size, pool_size)))
    model.add(conv.Convolution2D(n_filters, conv_size, conv_size,
                                 border_mode='valid',))
    model.add(core.Activation('relu'))
    model.add(conv.MaxPooling2D(pool_size=(pool_size, pool_size)))
    model.add(core.Dropout(dropout))
    model.add(core.Flatten())
    model.add(core.Dense(hidden_layer_size))
    model.add(core.Activation('sigmoid'))
    model.add(core.Dense(1))
    model.add(core.Activation('sigmoid'))
    model.compile(loss='binary_crossentropy', optimizer='adadelta')

    model_json = model.to_json()

    if out_path is not None:
        with open(out_path, 'w') as f:
            f.write(model_json)

    return model_json


def _populate_parser(parser):
    parser.description = 'Compiles a convolutional neural network to JSON.'
    parser.add_argument('--out_path', help='path to output model JSON',
                        default='data/model.json')
    parser.add_argument('--n_filters', help='number of convolutional filters',
                        default=32, type=int)
    parser.add_argument('--conv_size', help='size of convolutional filters',
                        default=4, type=int)
    parser.add_argument('--pool_size', help='size of max pool',
                        default=2, type=int)
    parser.add_argument('--dropout', help='dropout percentage',
                        default=0.25, type=float)
    parser.add_argument('--hidden_layer_size', help='hidden layer size',
                        default=64, type=int)
    parser.add_argument('--patch_size', help='size of image patches',
                        default=32, type=int)


def _main(args):
    main(args.n_filters, args.conv_size, args.pool_size, args.dropout,
         args.hidden_layer_size, args.patch_size, out_path=args.out_path)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    _populate_parser(parser)
    args = parser.parse_args()
    _main(args)



"""Setup for crowdastro.

Based on the setup script from https://github.com/pypa/sampleproject.

Matthew Alger
The Australian National University
2016
"""

from setuptools import setup, find_packages
from os import path

from crowdastro import __description__, __version__

here = path.abspath(path.dirname(__file__))

# Load README for long descriptions.
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='crowdastro',
    version=__version__,
    description=__description__,
    long_description=long_description,
    url='https://github.com/chengsoonong/crowdastro',
    # Setup scripts don't support multiple authors, so this should be the main
    # author or the author that should be contacted regarding the module.
    author='Matthew Alger',
    author_email='matthew.alger@anu.edu.au',
    license='MIT',
    classifiers=[
        # https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Astronomy',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords='machine-learning radio astronomy classification',
    packages=['crowdastro'],
    # Should be kept in sync with requirements.txt.
    install_requires=[
        'astropy',
        'h5py',
        'keras',
        'matplotlib',
        'numpy',
        'pandas',
        'pymongo',
        'requests',
        'scikit-learn',
        'scipy',
    ],
    extras_require={
        'dev': ['scikit-image'],
        'test': [],
    },
    package_dir={
        'crowdastro': 'crowdastro',
    },
    package_data={
        'crowdastro': ['../README.rst', 'crowdastro.json'],
    },
    entry_points={
        'console_scripts': [
            'crowdastro = crowdastro.__main__:main',
        ],
    },
)
